# -*- coding:utf-8 -*-
"""
Description:
    Fixed8

Usage:
    from neocore.Fixed8 import Fixed8
"""

import math


class Fixed8:
    value = 0

    D = 100000000

    """docstring for Fixed8"""

    def __init__(self, number):
        self.value = number

    def GetData(self):
        return self.value

    @staticmethod
    def FD():
        return Fixed8(Fixed8.D)

    @staticmethod
    def FromDecimal(number):
        out = int(number * Fixed8.D)
        return Fixed8(out)

    @staticmethod
    def Satoshi():
        return Fixed8(1)

    @staticmethod
    def One():
        return Fixed8(Fixed8.D)

    @staticmethod
    def NegativeSatoshi():
        return Fixed8(-1)

    @staticmethod
    def Zero():
        return Fixed8(0)

    @staticmethod
    def TryParse(value, require_positive=False):
        val = None
        try:
            val = float(value)
        except Exception as e:
            pass
        if not val:
            try:
                val = int(value)
            except Exception as e:
                pass
        if val is not None:

            if require_positive and val < 0:
                return None

            return Fixed8(int(val * Fixed8.D))

        return None

    def __add__(self, other):
        return Fixed8(self.value + other.value)

    def __iadd__(self, other):
        return self.__add__(other)

    def __sub__(self, other):
        return Fixed8(self.value - other.value)

    def __isub__(self, other):
        return self.__sub__(other)

    def __mul__(self, other):
        return Fixed8(self.value * other.value)

    def __imul__(self, other):
        return self.__mul__(other)

    def __truediv__(self, other):
        return Fixed8(int(self.value / other.value))

    def __floordiv__(self, other):
        return Fixed8(self.value // other.value)

    def __itruediv__(self, other):
        return self.__truediv__(other)

    def __mod__(self, other):
        return Fixed8(int(self.value % other.value))

    def __imod__(self, other):
        return self.__mod__(other)

    def __pow__(self, power, modulo=None):
        return Fixed8(pow(self.value, power.value, modulo))

    def __neg__(self):
        return Fixed8(-1 * self.value)

    def __eq__(self, other):
        return self.value == other.value

    def __lt__(self, other):
        return self.value < other.value

    def __gt__(self, other):
        return self.value > other.value

    def __ge__(self, other):
        return self.value >= other.value

    def __le__(self, other):
        return self.value <= other.value

    def Ceil(self):
        return Fixed8(math.ceil(self.value / Fixed8.D) * Fixed8.D)

    def Floor(self):
        return Fixed8(math.floor(self.value / Fixed8.D) * Fixed8.D)

    def ToInt(self):
        return int(self.value / Fixed8.D)

    def ToString(self):
        return str(self.value / Fixed8.D)

    def ToNeoJsonString(self):
        strval = self.ToString()
        if strval[-2:] == '.0':
            return strval[:-2]
        return strval

    def __str__(self):
        return self.ToString()

    def Size(self):
        return 8



# -*- coding:utf-8 -*-
"""
Description:
    Cryptography Helper
Usage:
    from neocore.Cryptography.Helper import *
"""

from bitcoin import *
import os
import time
import random
import hashlib
import binascii
import base58

long = int

ADDRESS_VERSION = 23


def double_sha256(ba):
    """
    Perform two SHA256 operations on the input.

    Args:
        ba (bytes): data to hash.

    Returns:
        str: hash as a double digit hex string.
    """
    d1 = hashlib.sha256(ba)
    d2 = hashlib.sha256()
    d1.hexdigest()
    d2.update(d1.digest())
    return d2.hexdigest()


def pubkey_to_redeem(pubkey):
    """
    Convert the public key to the redeemscript format.

    Args:
        pubkey (bytes): public key.

    Returns:
        bytes: redeemscript.
    """
    return binascii.unhexlify(b'21' + pubkey + b'ac')


def redeem_to_scripthash(redeem):
    """
    Convert a redeem script to a script hash.

    Args:
        redeem (bytes):

    Returns:
        bytes: script hash.
    """
    return bin_hash160Bytes(redeem)


def scripthash_to_address(scripthash):
    """
    Convert a script hash to a public address.

    Args:
        scripthash (bytes):

    Returns:
        str: base58 encoded string representing the wallet address.
    """
    sb = bytearray([ADDRESS_VERSION]) + scripthash
    c256 = bin_dbl_sha256(sb)[0:4]
    outb = sb + bytearray(c256)
    return base58.b58encode(bytes(outb)).decode("utf-8")


def pubkey_to_pubhash(pubkey):
    """
    Convert a public key to a script hash.

    Args:
        pubkey (bytes):

    Returns:
        bytes: script hash.
    """
    return redeem_to_scripthash(pubkey_to_redeem(pubkey))


def bin_dbl_sha256(s):
    """
    Perform a double SHA256 operation on the input.

    Args:
        s(str): message to hash.

    Returns:
        bytes: hash.
    """
    return hashlib.sha256(hashlib.sha256(s).digest()).digest()


def bin_hash160Bytes(bts):
    """
    Get a hash of the provided message using the ripemd160 algorithm.

    Args:
        bts (str): message to hash.

    Returns:
        bytes: hash.
    """
    intermed = hashlib.sha256(bts).digest()
    return hashlib.new('ripemd160', intermed).digest()


def bin_hash160(string):
    """
    Get a hash of the provided message using the ripemd160 algorithm.

    Args:
        string (str): message to hash.

    Returns:
        str: hash as a double digit hex string.
    """
    intermed = hashlib.sha256(string).digest()
    return hashlib.new('ripemd160', intermed).hexdigest()


def base256_encode(n, minwidth=0):  # int/long to byte array
    """
    Encode the input with base256.

    Args:
        n (int): input value.
        minwidth: minimum return value length.

    Raises:
        ValueError: if a negative number is provided.

    Returns:
        bytearray:
    """
    if n > 0:
        arr = []
        while n:
            n, rem = divmod(n, 256)
            arr.append(rem)
        b = bytearray(reversed(arr))
    elif n == 0:
        b = bytearray(b'\x00')
    else:
        raise ValueError("Negative numbers not supported")

    if minwidth > 0 and len(b) < minwidth:  # zero padding needed?
        padding = (minwidth - len(b)) * b'\x00'
        b = bytearray(padding) + b
    b.reverse()

    return b


def xor_bytes(a, b):
    """
    XOR on two bytes objects

    Args:
        a (bytes): object 1
        b (bytes): object 2

    Returns:
        bytes: The XOR result
    """
    assert isinstance(a, bytes)
    assert isinstance(b, bytes)
    assert len(a) == len(b)
    res = bytearray()
    for i in range(len(a)):
        res.append(a[i] ^ b[i])
    return bytes(res)



from neocore.UIntBase import UIntBase


class UInt160(UIntBase):
    def __init__(self, data=None):
        super(UInt160, self).__init__(num_bytes=20, data=data)

    @staticmethod
    def ParseString(value):
        if value[0:2] == '0x':
            value = value[2:]
        if not len(value) == 40:
            raise Exception("Invalid UInt160 Format: %s chars != 40 chars" % len(value))
        reversed_data = bytearray.fromhex(value)
        reversed_data.reverse()
        return UInt160(data=reversed_data)
